@@ -720,9 +720,11 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
 }
 
 void Control::addRecent (const String &url) {
+    if (url == "Playlist://")
+        return;
     recents->defer ();
     NodePtr c = recents->firstChild ();
-    if (url != String ("Playlist://") && (!c || c->mrl()->src != url)) {
+    if (!c || c->mrl()->src != url) {
         recents->insertBefore (new Recent (recents, m_app, url), c);
         c = recents->firstChild ()->nextSibling ();
         int count = 1;
@@ -1061,6 +1063,7 @@ gboolean cb_keyPressed (GtkWidget *, GdkEventKey * e, Application *app) {
         case GDK_F6: //HILDON_HARDKEY_FULLSCREEN:
             //if (app->control->process ()->playing ()) {
              //   int w, h;
+            VIEWAREA(app)->prepareFullscreenToggle ();
             if (!app->fullscreen) {
                 if (CONTROL(app)->playlist_data->document &&
                         CONTROL(app)->playlist_data->document->active ())
