@@ -29,8 +29,6 @@
 #include "kmplayertypes.h"
 #include "kmplayershared.h"
 
-typedef struct _cairo_surface cairo_surface_t;
-
 namespace KMPlayer {
 
 class Document;
@@ -38,7 +36,6 @@ class Node;
 class TextNode;
 class Posting;
 class Mrl;
-class Surface;
 class ElementPrivate;
 class Visitor;
 class MediaInfo;
@@ -169,6 +166,7 @@ class KMPLAYER_EXPORT TreeNode : public ListNodeBase <T> {
 public:
     virtual ~TreeNode () {}
 
+    virtual void insertBefore (typename Item<T>::SharedType c, typename Item<T>::SharedType b);
     virtual void appendChild (typename Item<T>::SharedType c);
     virtual void removeChild (typename Item<T>::SharedType c);
 
@@ -230,6 +228,7 @@ enum MessageType
     MsgMediaReady,
     MsgMediaUpdated,
     MsgEventPostponed,
+    MsgSurfaceBoundsUpdate,      // bool parent surface resized
     MsgSurfaceUpdate,
     MsgSurfaceAttach,
     MsgStateFreeze,
@@ -275,8 +274,6 @@ typedef List<NodeRefItem> NodeRefList;       // ref nodes, eg. event receivers
 typedef Item<NodeRefList>::SharedType NodeRefListPtr;
 typedef Item<NodeRefList>::WeakType NodeRefListPtrW;
 ITEM_AS_POINTER(KMPlayer::NodeRefList)
-typedef Item<Surface>::SharedType SurfacePtr;
-typedef Item<Surface>::WeakType SurfacePtrW;
 
 /*
  * Weak ref of the receivers list from signaler and the listener node
@@ -501,13 +498,11 @@ private:
  */
 class RefNode : public Node {
 public:
-    RefNode (NodePtr & d, NodePtr ref);
-    virtual const char * nodeName () const { return (const char *) tag_name; }
+    RefNode (NodePtr &doc, NodePtr ref) : Node (doc), ref_node (ref) {}
+    virtual const char *nodeName () const { return "#ref"; }
     NodePtr refNode () const { return ref_node; }
-    void setRefNode (const NodePtr ref);
 protected:
     NodePtrW ref_node;
-    String tag_name;
 };
 
 template <class T>
@@ -525,7 +520,10 @@ protected:
     void parseParam (const TrieString &, const String &);
     unsigned int cached_ismrl_version;
     PlayType cached_play_type;
+
 public:
+    enum { SingleMode = 0, WindowMode };
+
     ~Mrl ();
     PlayType playType ();
     /*
@@ -554,7 +552,7 @@ public:
     Single height;
     float aspect;
     int repeat;
-    enum { SingleMode = 0, WindowMode } view_mode;
+    unsigned char view_mode;
     bool resolved;
     bool bookmarkable;
     bool audio_only;
@@ -591,31 +589,6 @@ public:
     virtual void enableRepaintUpdaters (bool enable, unsigned int off_time)=0;
 };
 
-class KMPLAYER_NO_EXPORT Surface : public TreeNode <Surface> {
-public:
-    Surface (NodePtr node, const SRect & rect);
-    ~Surface();
-
-    virtual SurfacePtr createSurface (NodePtr owner, const SRect & rect) = 0;
-    virtual IRect toScreen (Single x, Single y, Single w, Single h) = 0;
-    virtual void resize (const SRect & rect) = 0;
-    virtual void repaint () = 0;
-    virtual void repaint (const SRect &rect) = 0;
-    void remove ();                // remove from parent, mark ancestors dirty
-    void markDirty ();             // mark this and ancestors dirty
-
-    NodePtrW node;
-    SRect bounds;                  // bounds in in parent coord.
-    float xscale, yscale;          // internal scaling
-    unsigned int background_color; // rgba background color
-    bool dirty;                    // a decendant is removed
-    cairo_surface_t *surface;
-protected:
-    Surface (const SRect & rect);
-};
-
-ITEM_AS_POINTER(KMPlayer::Surface)
-
 /*
  *  A generic type for posting messages
  **/
@@ -957,6 +930,25 @@ inline void TreeNode<T>::appendChild (typename Item<T>::SharedType c) {
     c->m_parent = Item<T>::m_self;
 }
 
+template <class T>
+inline void TreeNode<T>::insertBefore (
+        typename Item<T>::SharedType c, typename Item<T>::SharedType b) {
+    if (!b) {
+        appendChild (c);
+    } else {
+        if (b->m_prev) {
+            b->m_prev->m_next = c;
+            c->m_prev = b->m_prev;
+        } else {
+            c->m_prev = 0L;
+            m_first_child = c;
+        }
+        b->m_prev = c;
+        c->m_next = b;
+        c->m_parent = Item<T>::m_self;
+    }
+}
+
 template <class T>
 inline void TreeNode<T>::removeChild (typename Item<T>::SharedType c) {
     if (c->m_prev) {
