@@ -429,38 +429,47 @@ static void restoreEditMode (Application * app, PlayListData * pld) {
         (GTK_WIDGET(app->menu_item_playlist_edit), !!pld->document);
 }
 
+static void activateTopNode (GtkTreeView *tree_view, GtkTreePath *path,
+        PlayListData *pld, NodePtr node, Application *app) {
+    bool need_update = false;
+    GtkTreeModel * model = gtk_tree_view_get_model (tree_view);
+    GtkTreeIter iter;
+    if (node->isPlayable () && !node->active ()) {
+        restoreEditMode (app, pld); // cb_selection_changed is skipped
+        node->activate ();
+        need_update = !node->isPlayable ();
+    } else if (!node->isPlayable ()) {
+        if (pld->updated_document_version !=
+                node->document ()->m_tree_version) {
+            need_update = true;
+        } else {
+            gtk_tree_model_get_iter (model, &iter, path);
+            need_update = !gtk_tree_model_iter_has_child (model, &iter);
+        }
+    }
+    if (need_update) {
+        pld->updated_document = NULL;
+        updateTree (app, pld->id);
+        gtk_tree_model_get_iter (model, &iter, path);
+        if (gtk_tree_model_iter_has_child (model, &iter)) {
+            if (!gtk_tree_view_row_expanded (tree_view, path))
+                gtk_tree_view_expand_row (tree_view, path, FALSE);
+            else
+                gtk_tree_view_collapse_row (tree_view, path);
+        }
+    }
+}
+
 gboolean cb_treeViewClicked (GtkWidget *view, GdkEventButton *evt, Application * app) {
-    GtkTreePath * path;
-    GtkTreeView * tree_view = GTK_TREE_VIEW (view);
+    GtkTreePath *path;
+    GtkTreeView *tree_view = GTK_TREE_VIEW (view);
     if (gtk_tree_view_get_path_at_pos (tree_view, (int)evt->x, (int)evt->y,
                 &path, 0l, 0L, 0L)) {
-        PlayListData * pld = getListData (CONTROL(app), path);
+        PlayListData *pld = getListData (CONTROL(app), path);
         NodePtr node = getNode (pld, path);
     debugLog() << "tree clicked" << endl;
         if (node && gtk_tree_path_get_depth (path) == 1) {
-            bool need_update = false;
-            GtkTreeModel * model = gtk_tree_view_get_model (tree_view);
-            GtkTreeIter iter;
-            if (node->isPlayable () && !node->active ()) {
-                restoreEditMode (app, pld); // cb_selection_changed is skipped
-                node->activate ();
-                need_update = !node->isPlayable ();
-            } else if (!node->isPlayable ()) {
-                gtk_tree_model_get_iter (model, &iter, path);
-                need_update = !gtk_tree_model_iter_has_child (model, &iter);
-            }
-            if (need_update) {
-                pld->updated_document = NULL;
-                updateTree (app, pld->id);
-                gtk_tree_model_get_iter (model, &iter, path);
-                if (gtk_tree_model_iter_has_child (model, &iter)) {
-                    if (!gtk_tree_view_row_expanded (tree_view, path)) {
-                        gtk_tree_view_expand_to_path (tree_view, path);
-                    } else {
-                        gtk_tree_view_collapse_row (tree_view, path);
-                    }
-                }
-            }
+            activateTopNode (tree_view, path, pld, node, app);
         }
         gtk_tree_path_free (path);
     }
@@ -501,17 +510,19 @@ void cb_selection_changed (GtkTreeSelection *selection, Application * app) {
 
 void cb_row_activated (GtkTreeView * tree_view, GtkTreePath *path,
         GtkTreeViewColumn *arg2, Application * app) {
-    debugLog() << "row activated" << endl;
-    if (gtk_tree_path_get_depth (path) == 1)
-        return; // handled by clicked
-    PlayListData * pld = getListData (CONTROL(app), path);
+    PlayListData *pld = getListData (CONTROL(app), path);
     NodePtr node = getNode (pld, path);
-    if (pld->id == 0 || pld->edit_mode)
-        app_set_info_content (app, node
-                ? (pld->edit_mode ? node->innerXML () : node->innerText ())
-                : "", pld->edit_mode);
-    if (!pld->edit_mode && (pld->id == 0 || node->mrl ()))
-        jump (node.ptr (), CONTROL(app));
+    debugLog() << "row activated" << endl;
+    if (gtk_tree_path_get_depth (path) == 1) {
+        activateTopNode (tree_view, path, pld, node, app);
+    } else {
+        if (pld->id == 0 || pld->edit_mode)
+            app_set_info_content (app, node
+                    ? (pld->edit_mode ? node->innerXML () : node->innerText ())
+                    : "", pld->edit_mode);
+        if (!pld->edit_mode && (pld->id == 0 || node->mrl ()))
+            jump (node.ptr (), CONTROL(app));
+    }
 }
 
 void KMPlayer::rowSelected (Application *app) {
