@@ -154,8 +154,12 @@ static gboolean cb_scheduledUpdateAspects (void * p) {
 }
 
 KMPLAYER_NO_EXPORT void Process::updateAspects () {
-    if (m_notify)
-        m_notify->setAspect (m_height > 0 ? 1.0 * m_width / m_height : 0);
+    if (m_notify) {
+        float aspect = m_aspect;
+        if (aspect < 0.01)
+            aspect = m_height > 0 ? 1.0 * m_width / m_height : 0;
+        m_notify->setAspect (aspect);
+    }
 }
 
 static gboolean cb_scheduledStateChanged (void * p) {
@@ -183,6 +187,7 @@ KMPLAYER_NO_EXPORT bool Process::play (NodePtr _mrl) {
     no_video = m ? m->audio_only : false;
     m_position = m_length = 0;
     m_width = m_height = 0;
+    m_aspect = 0.0;
     return false;
 }
 
@@ -278,6 +283,24 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     if (mrl->repeat > 0)
         cmdline += String (" -loop ") + String::number (mrl->repeat+1);
     cmdline += Char (' ');
+
+    for (NodePtr n = mrl; n; n = n->parentNode ()) {
+        if (n->id != id_node_group_node && n->id != id_node_playlist_item)
+            break;
+        String plops = convertNode <Element> (n)->getAttribute ("mplayeropts");
+        if (!plops.isNull ()) {
+            gchar ** list = g_strsplit ((const char *) plops, " ", -1);
+            for (int i = 0; list [i]; i++) {
+                char *quoted_opt = g_shell_quote (list [i]);
+                cmdline += quoted_opt;
+                cmdline += Char (' ');
+                g_free (quoted_opt);
+            }
+            g_strfreev (list);
+            break;
+        }
+    }
+
     char *quoted = g_shell_quote ((const char *) m_url);
     cmdline += String (quoted);
     g_free (quoted);
@@ -425,11 +448,14 @@ KMPLAYER_NO_EXPORT bool MPlayer::brightness (int pos, bool absolute) {
 }
 
 KMPLAYER_NO_EXPORT void MPlayer::setVideoWindow (int x, int y, int w, int h) {
-    debugLog() << "setVideoWindow " << w << " " << h << endl;
-    if (x < 0 || y < 0 || w <= 0 || h <= 1 /* hack !!*/ ||
+    debugLog() << "setVideoWindow " << x << "," << y << " " << w << "x" << h << endl;
+    if (x + w < 0 || y + h < 0 || w <= 0 || h <= 1 /* hack !!*/ ||
                 x > 800 || y > 480 || w > 800 || h > 480)
         return;
-    m_rect = Rect (x, y, w, h);
+    Rect rect = Rect (x, y, w, h);
+    if (rect == m_rect)
+        return;
+    m_rect = rect;
     if (!hasVideo ())
         return;
     char buf[64];
@@ -563,6 +589,11 @@ KMPLAYER_NO_EXPORT void MPlayer::readEvent (GIOChannel * src) {
                             m_height = (int) strtof (tok + 16, 0L);
                             if (m_width > 0)
                                 updateAspects ();
+                        } else if (!strncmp (tok + 3, "VIDEO_ASPECT", 12)) {
+                            // convert in C locale
+                            m_aspect = (float) String (tok + 16).toDouble ();
+                            if (m_aspect > 0.01)
+                                updateAspects ();
                         }
                     } else {
                         char * icy = strstr (tok, "ICY Info");
