@@ -1433,6 +1433,7 @@ KDE_NO_CDTOR_EXPORT ViewArea::ViewArea ()
    aspect (0.0),
    widget (0L),
    video (0L),
+   video_input (0L),
    surface (new Surface (this)),
    last_repaint_time (0),
    next_repaint_time (0),
@@ -1572,6 +1573,7 @@ void ViewArea::setAudioVideoGeometry (const IRect &rect, unsigned int * bg_color
         debugLog() << "ViewArea::setAudioVideoGeometry " << x << "," << y << " "
             << w << "x" << h << endl;
         gdk_window_move_resize (video, x, y, w, h);
+        gdk_window_move_resize (video_input, x, y, w, h);
         gdk_window_set_background (video, (GdkColor *) (GColor &) color);
         gdk_window_clear (video);
     }
@@ -1741,6 +1743,15 @@ KDE_NO_EXPORT int ViewArea::height () const {
     return h;
 }
 
+void ViewArea::prepareFullscreenToggle () {
+    // MPlayer+omapfb stays black in window mode if having the focus
+    Window w = GDK_DRAWABLE_XID (GDK_DRAWABLE (widget->window));
+    Display *dpy = gdk_x11_display_get_xdisplay(gtk_widget_get_display(widget));
+
+    XSetInputFocus (dpy, w, RevertToParent, CurrentTime);
+    XFlush (dpy);
+}
+
 static void setXSelectInput (Display *dpy, Window wid, long mask) {
     XWindowAttributes attr;
     XGetWindowAttributes (dpy, wid, &attr);
@@ -1756,7 +1767,39 @@ static void setXSelectInput (Display *dpy, Window wid, long mask) {
     }
 }
 
-static bool isChildWindow (Display *dpy, Window p, Window c) {
+static bool findWindowAt (Display *dpy, Window wid, long mask, int *x, int *y,
+        Window *window, Window *sub_window) {
+    XWindowAttributes attr;
+    XGetWindowAttributes (dpy, wid, &attr);
+    Window r, p, *c;
+    unsigned int nr;
+    *window = 0;
+    if (XQueryTree (dpy, wid, &r, &p, &c, &nr)) {
+        for (int i = nr-1; i >= 0; --i) {
+            XWindowAttributes cattr;
+            XGetWindowAttributes (dpy, c[i], &cattr);
+            if (*x >= cattr.x && *x <= cattr.x + cattr.width &&
+                    *y >= cattr.y && *y <= cattr.y + cattr.height) {
+                int x1 = *x - cattr.x;
+                int y1 = *y - cattr.y;
+                *sub_window = c[i];
+                if (findWindowAt(dpy, c[i], mask, &x1, &y1, window, sub_window))
+                {
+                    *x = x1;
+                    *y = y1;
+                    break;
+                }
+            }
+        }
+        XFree (c);
+    }
+    if (!*window && attr.all_event_masks & mask)
+        *window = wid;
+    return *window;
+
+}
+
+static int getChildWindowDepth (Display *dpy, Window p, Window c, int depth) {
     Window root = 0;
     Window parent = 0;
     Window *children;
@@ -1770,17 +1813,20 @@ static bool isChildWindow (Display *dpy, Window p, Window c) {
         c = parent;
     }
     return parent == p;*/
-    bool found = false;
-    if (XQueryTree (dpy, p, &root, &parent, &children, &nr)) {
-        for (int i = 0; i < nr; ++i) {
-            found = children[i] == c || isChildWindow (dpy, children[i], c);
-            if (found)
-                break;
+    int d = -1;
+    if ( p == c) {
+        d = depth;
+    } else if (XQueryTree (dpy, p, &root, &parent, &children, &nr)) {
+        for (int i = 0; d < 0 && i < nr; ++i) {
+            if (children[i] == c )
+                d = depth + 1;
+            else
+                d = getChildWindowDepth (dpy, children[i], c, depth + 1);
         }
         if (nr)
             XFree (children);
     }
-    return found;
+    return d;
 }
 
 static
@@ -1802,8 +1848,12 @@ GdkFilterReturn videoWidgetFilter (GdkXEvent *xe, GdkEvent *e, Application *a) {
                     XKeycodeToKeysym (event->xany.display, keyevent.keycode, 0) == XK_F4)
                 return GDK_FILTER_CONTINUE; //GDK_FILTER_REMOVE;
             Window v = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoWindow()));
-            if (event->xany.window == v ||
-                    isChildWindow(event->xany.display, v, event->xkey.window)) {
+            Window i = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoInputWindow()));
+            int depth = event->xany.window == i
+                ? 0
+                : getChildWindowDepth (event->xany.display, v,
+                        event->xkey.window, 0);
+            if (depth >= 0) {
                 keyevent.window =
                     GDK_DRAWABLE_XID (GDK_DRAWABLE (view_area->viewWidget ()->window));
                 XSendEvent (event->xany.display, keyevent.window, true,
@@ -1814,43 +1864,72 @@ GdkFilterReturn videoWidgetFilter (GdkXEvent *xe, GdkEvent *e, Application *a) {
         }
         case MapNotify: {
             Window v = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoWindow()));
-            if (!event->xmap.override_redirect &&
-                 (event->xany.window == v ||
-                  isChildWindow (event->xany.display, v, event->xmap.window))) {
-                setXSelectInput (event->xany.display,
-                        event->xmap.window,
-                        KeyPressMask|ExposureMask|SubstructureNotifyMask);
+            if (!event->xmap.override_redirect) {
+                int depth = getChildWindowDepth (event->xany.display,
+                        v, event->xmap.window, 0);
+                if (depth >= 0) {
+                    setXSelectInput (event->xany.display,
+                            event->xmap.window,
+                            KeyPressMask|ExposureMask|SubstructureNotifyMask);
+                }
             }
             break;
         }
+        case ConfigureNotify:
+            if (view_area->videoInputWindow ()) {
+                Window v = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoWindow()));
+                Window i = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoInputWindow()));
+                if (event->xconfigure.window == v &&
+                        event->xconfigure.above == i)
+                    XRaiseWindow (event->xany.display, i);
+            }
+            break;
         case ButtonPress: {
+            Window i = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoInputWindow()));
             Window v = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoWindow()));
             Window w = GDK_DRAWABLE_XID (GDK_DRAWABLE (view_area->viewWidget ()->window));
             Rect vr = view_area->videoGeometry ();
             if (a->fullscreen &&
                     (event->xany.window == w &&
                      event->xbutton.x > 720 && event->xbutton.y < 80) ||
-                    (event->xany.window == v &&
+                    (event->xany.window == i &&
                      event->xbutton.x + vr.x () > 720 &&
                      event->xbutton.y + vr.y () < 80)) {
                 if (!view_area->controls_timer)
                     updatePlayTree (a);
                 view_area->controlsVisible (a, true);
-            } else if (event->xany.window == w || event->xany.window == v) {
-                if (event->xbutton.time - down_time < 1000 &&
+            } else if (event->xany.window == w || event->xany.window == i) {
+                if (event->xbutton.time - down_time < 600 &&
                         abs (down_x - event->xbutton.x) < 80 ) {
+                    view_area->prepareFullscreenToggle ();
                     if (a->fullscreen)
                         app_normal_screen (a);
                     else
                         app_full_screen (a);
                     down_time = 0;
+                    return GDK_FILTER_REMOVE;
                 } else {
                     down_x = event->xbutton.x;
                     down_time = event->xbutton.time;
                 }
             }
-            break;
         }
+        // fall through
+        case ButtonRelease:
+            if (event->xany.window == GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoInputWindow()))) {
+                XButtonEvent evt = event->xbutton;
+                int evt_mask = event->xany.type == ButtonRelease
+                ? ButtonReleaseMask : ButtonPressMask;
+                if (findWindowAt (event->xany.display,
+                            GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoWindow())),
+                            evt_mask,
+                            &evt.x, &evt.y, &evt.window, &evt.subwindow)) {
+                    XSendEvent (event->xany.display, evt.window, False,
+                            evt_mask, (XEvent *) &evt);
+                    XFlush (event->xany.display);
+                    return GDK_FILTER_REMOVE;
+                }
+            }
     }
     return GDK_FILTER_CONTINUE;
 }
@@ -1961,6 +2040,7 @@ KDE_NO_EXPORT void ViewArea::setEventFiltering (Application *app, bool on) {
 KDE_NO_EXPORT void ViewArea::setOutputWidget (Application *app, GtkWidget * w) {
     widget = w;
     gdk_window_clear (w->window);
+    gdk_window_focus (widget->window, CurrentTime);
     if (surface->surface) {
         cairo_surface_destroy (surface->surface);
         surface->surface = 0L;
@@ -1982,14 +2062,23 @@ KDE_NO_EXPORT void ViewArea::setOutputWidget (Application *app, GtkWidget * w) {
         gint mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
         video = gdk_window_new (w->window, &attributes, mask);
         gdk_window_show (video);
+
+        attributes.wclass = GDK_INPUT_ONLY;
+        attributes.event_mask = gtk_widget_get_events (w) |
+            GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK;
+        attributes.event_mask &= ~GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK;
+        mask = GDK_WA_X | GDK_WA_Y | GDK_WA_WMCLASS;
+        video_input = gdk_window_new (w->window, &attributes, mask);
+        gdk_window_show (video_input);
+
         setAudioVideoGeometry (IRect (-100, -100, 50, 50), 0L);
 
         debugLog() << "video window "  << (int)GDK_DRAWABLE_XID (GDK_DRAWABLE (video)) << " " << endl;
 
     } else {
         gdk_window_reparent (video, widget->window, m_av_geometry.x(), m_av_geometry.y());
+        gdk_window_reparent (video_input, widget->window, m_av_geometry.x(), m_av_geometry.y());
     }
-    gdk_window_focus (widget->window, CurrentTime);
     //resizeEvent ();
 }
 
