@@ -58,11 +58,13 @@ public:
     void paintEvent (const IRect &);
     bool timeoutPaintEvent ();
     void controlsVisible (Application *app, bool show);
+    void prepareFullscreenToggle ();
     KDE_NO_EXPORT Rect videoGeometry () const { return m_av_geometry; }
     int width () const;
     int height () const;
-    GtkWidget * viewWidget () const { return widget; }
-    GdkWindow * videoWindow () const { return video; }
+    GtkWidget *viewWidget () const { return widget; }
+    GdkWindow *videoWindow () const { return video; }
+    GdkWindow *videoInputWindow () const { return video_input; }
     Mrl *videoMrl () { return video_node ? video_node->mrl () : NULL; }
     void setOutputWidget (Application *, GtkWidget * w);
     void setControlsOverlap (bool b) { m_controls_overlap = b; }
@@ -77,8 +79,9 @@ private:
     Rect m_av_geometry;
     float aspect;
     IRect m_repaint_rect;
-    GtkWidget * widget;
-    GdkWindow * video;
+    GtkWidget *widget;
+    GdkWindow *video;
+    GdkWindow *video_input;
     SurfacePtr surface;
     NodePtrW video_node;
     unsigned int last_repaint_time;
